import React, { useMemo, useState } from 'react';
import { Chart, chartSchema } from '@baseline/types/chart';
import { Formik } from 'formik';
import * as Yup from 'yup';
import {
  Button,
  FormFeedback,
  FormGroup,
  Input,
  Label,
  Modal,
  ModalBody,
  ModalFooter,
  ModalHeader,
} from 'reactstrap';
import { createChart } from '@baseline/client-api/chart';
import { onChartCreated } from '../../../../swr/chart';

const CreateChartModal = (): JSX.Element => {
  const [isOpen, setIsOpen] = useState(false);
  const toggle = () => setIsOpen(!isOpen);

  const initialValues: Chart = useMemo(
    () => ({
      chartId: undefined,
      parentChart: 'NO_PARENT',
      name: '',
      md5: '',
      sha256: '',
      resourceUri: '',
      comment: '',
    }),
    [],
  );

  return (
    <Formik
      initialValues={initialValues}
      enableReinitialize
      validationSchema={chartSchema}
      onSubmit={async (values, { setSubmitting, resetForm }) => {
        setSubmitting(true);
        const newChart = await createChart(values);
        await onChartCreated(newChart);

        resetForm();
        setIsOpen(false);
      }}
    >
      {({
        values,
        handleSubmit,
        isValid,
        isSubmitting,
        resetForm,
        errors,
        setFieldValue,
        handleBlur,
      }) => (
        <>
          <Button
            onClick={() => {
              resetForm();
              setIsOpen(true);
            }}
          >
            Create Chart
          </Button>
          <Modal
            centered
            isOpen={isOpen}
            toggle={() => {
              toggle();
              resetForm();
            }}
          >
            <ModalHeader toggle={toggle}>Create Chart</ModalHeader>
            <ModalBody>
              <FormGroup>
                <Label htmlFor="name">Name</Label>
                <Input
                  name="name"
                  id="name"
                  value={values.name}
                  onChange={(e) => void setFieldValue('name', e.target.value)}
                  onBlur={handleBlur}
                  className={errors.name ? 'is-invalid' : ''}
                />
                <FormFeedback>{errors.name}</FormFeedback>
              </FormGroup>
              <FormGroup>
                <Label htmlFor="parentChart">Parent Chart</Label>
                <Input
                  name="parentChart"
                  id="parentChart"
                  value={values.parentChart}
                  onChange={(e) =>
                    void setFieldValue('parentChart', e.target.value)
                  }
                  onBlur={handleBlur}
                  className={errors.parentChart ? 'is-invalid' : ''}
                  disabled
                  placeholder="This will be a dropdown at some point"
                />
                <FormFeedback>{errors.parentChart}</FormFeedback>
              </FormGroup>
              <FormGroup>
                <Label htmlFor="md5">MD5</Label>
                <Input
                  name="md5"
                  id="md5"
                  value={values.md5}
                  onChange={(e) => void setFieldValue('md5', e.target.value)}
                  onBlur={handleBlur}
                  className={errors.md5 ? 'is-invalid' : ''}
                />
                <FormFeedback>{errors.md5}</FormFeedback>
              </FormGroup>
              <FormGroup>
                <Label htmlFor="sha256">SHA256</Label>
                <Input
                  name="sha256"
                  id="sha256"
                  value={values.sha256}
                  onChange={(e) => void setFieldValue('sha256', e.target.value)}
                  onBlur={handleBlur}
                  className={errors.sha256 ? 'is-invalid' : ''}
                />
                <FormFeedback>{errors.sha256}</FormFeedback>
              </FormGroup>
              <FormGroup>
                <Label htmlFor="resourceUri">Resource URI</Label>
                <Input
                  name="resourceUri"
                  id="resourceUri"
                  value={values.resourceUri}
                  onChange={(e) =>
                    void setFieldValue('resourceUri', e.target.value)
                  }
                  onBlur={handleBlur}
                  className={errors.resourceUri ? 'is-invalid' : ''}
                />
                <FormFeedback>{errors.resourceUri}</FormFeedback>
              </FormGroup>
              <FormGroup>
                <Label htmlFor="comment">Comment</Label>
                <textarea
                  name="comment"
                  id="comment"
                  value={values.comment}
                  onChange={(e) =>
                    void setFieldValue('comment', e.target.value)
                  }
                  onBlur={handleBlur}
                  className={`${
                    errors.comment ? 'is-invalid' : ''
                  } form-control`}
                />
                <FormFeedback>{errors.comment}</FormFeedback>
              </FormGroup>
            </ModalBody>
            <ModalFooter>
              <Button color="secondary" onClick={() => setIsOpen(false)}>
                Cancel
              </Button>
              <Button
                onClick={() => {
                  void handleSubmit();
                }}
                disabled={!isValid || isSubmitting}
                color="primary"
              >
                Create
              </Button>
            </ModalFooter>
          </Modal>
        </>
      )}
    </Formik>
  );
};

export default CreateChartModal;
